<?php
/**
 * @copyright Copyright (c) 2017 Thomas Citharel <nextcloud@tcit.fr>
 *
 * @author Thomas Citharel <nextcloud@tcit.fr>
 *
 * @license GNU AGPL version 3 or any later version
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

namespace OCA\DropAccount\Controller;

use DateTime;
use Exception;
use OCA\DropAccount\AppInfo\Application;
use OCA\DropAccount\BackgroundJob\DeleteAccountData;
use OCA\DropAccount\MissingEmailException;
use OCA\DropAccount\Service\ConfirmationService;
use OCA\DropAccount\Service\DeletionCapabilityService;
use OCA\DropAccount\Service\MailerService;
use OCP\AppFramework\Http;
use OCP\AppFramework\Http\DataResponse;
use OCP\AppFramework\OCSController;
use OCP\BackgroundJob\IJobList;
use OCP\IConfig;
use OCP\IL10N;
use OCP\IRequest;
use OCP\IUser;
use OCP\IUserSession;
use OCP\PreConditionNotMetException;
use Psr\Log\LoggerInterface;

class APIController extends OCSController {
	private IUserSession $userSession;
	private LoggerInterface $logger;
	private IJobList $jobList;
	private IConfig $config;
	private ConfirmationService $confirmationService;
	private IL10N $l10n;
	private MailerService $mailerService;
	private DeletionCapabilityService $deletionCapabilityService;

	public function __construct(string              $appName,
		IRequest            $request,
		IUserSession        $userSession,
		LoggerInterface     $logger,
		IConfig             $config,
		IL10N               $l10n,
		IJobList            $jobList,
		ConfirmationService $confirmationService,
		MailerService       $mailerService,
		DeletionCapabilityService $deletionCapabilityService,
	) {
		parent::__construct($appName, $request);
		$this->userSession = $userSession;
		$this->logger = $logger;
		$this->config = $config;
		$this->l10n = $l10n;
		$this->jobList = $jobList;
		$this->confirmationService = $confirmationService;
		$this->mailerService = $mailerService;
		$this->deletionCapabilityService = $deletionCapabilityService;
	}

	/**
	 *
	 * Trigger the deletion of the current user account
	 *
	 * @NoAdminRequired
	 * @PasswordConfirmationRequired
	 * @return DataResponse<Http::STATUS_CREATED, array{message: string}, array{}>|DataResponse<Http::STATUS_ACCEPTED, array{}, array{}>|DataResponse<Http::STATUS_UNAUTHORIZED, array{message: string}, array{}>|DataResponse<Http::STATUS_FORBIDDEN, array{message: string|null}, array{}>|DataResponse<Http::STATUS_BAD_REQUEST, array{message: string}, array{}>|DataResponse<Http::STATUS_INTERNAL_SERVER_ERROR, array{message: string}, array{}>
	 * 201: A confirmation by email is required. An email has been sent to the user
	 * 202: The user has been marked for deletion
	 * 400: The user has no email set up in it's account to send the confirmation email to
	 * 401: Unable to get user from session
	 * 403: The user was not allowed to be deleted
	 * 500: An exception was thrown when sending the user a confirmation email
	 */
	public function delete(): DataResponse {
		$user = $this->userSession->getUser();

		if (!$user) {
			return new DataResponse(['message' => $this->l10n->t('User-Session unexpectedly expired')], Http::STATUS_UNAUTHORIZED);
		}

		[$canBeDeleted, $reason] = $this->deletionCapabilityService->canUserBeDeleted($user);
		if (!$canBeDeleted) {
			return new DataResponse([
				'message' => $reason
			], Http::STATUS_FORBIDDEN);
		}

		if ($this->config->getAppValue(Application::APP_NAME, 'requireConfirmation', 'no') === 'yes') {
			try {
				$token = $this->confirmationService->sendConfirmationEmail($user);
				$this->config->setUserValue($user->getUID(), Application::APP_NAME, 'delete_token', $token);
				return new DataResponse([
					'message' => $this->l10n->t('Successfully sent email')
				], Http::STATUS_CREATED);
			} catch (MissingEmailException) {
				return new DataResponse([
					'message' => $this->l10n->t('You have no email set up into your account')
				], Http::STATUS_BAD_REQUEST);
			} catch (Exception $e) {
				$this->logger->error('Unknown exception when sending email for purging user', ['exception' => $e]);
				return new DataResponse([
					'message' => $this->l10n->t('Unexpected error sending email. Please contact your administrator.')
				], Http::STATUS_INTERNAL_SERVER_ERROR);
			}
		} else {
			$this->performDelete($user);
			return new DataResponse([], Http::STATUS_ACCEPTED);
		}
	}

	/**
	 * @param IUser $user
	 */
	private function performDelete(IUser $user): void {
		$this->config->deleteUserValue($user->getUID(), Application::APP_NAME, 'delete_token');
		try {
			$this->mailerService->sendFinalEmail($user);
		} catch (Exception) {
			// Do nothing, this is not blocking
		}
		$user->setEnabled(false);

		/**
		 * If we delay purge, just set the date when the user should be removed to a config user value
		 */
		if ($this->config->getAppValue(Application::APP_NAME, 'delayPurge', 'no') === 'yes') {
			try {
				$this->config->setUserValue($user->getUID(), Application::APP_NAME, 'markedForPurge', 'yes');
				$this->config->setUserValue($user->getUID(), Application::APP_NAME, 'purgeDate', (string)(new DateTime())->getTimestamp());
			} catch (PreConditionNotMetException) {
				// Do nothing, it will not happen, we don't give preconditions
			}
		} else {
			$this->jobList->add(DeleteAccountData::class, ['uid' => $user->getUID()]);
		}
		$this->userSession->logout();
	}
}
