<?php

declare(strict_types=1);
/**
 * @copyright Copyright (c) 2023 Thomas Citharel <nextcloud@tcit.fr>
 *
 * @author Thomas Citharel <nextcloud@tcit.fr>
 *
 * @license GNU AGPL version 3 or any later version
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 */

namespace OCA\DropAccount\Service;

use OCP\IGroupManager;
use OCP\IL10N;
use OCP\IUser;
use OCP\IUserManager;

class DeletionCapabilityService {

	private IUserManager $userManager;
	private IGroupManager $groupManager;
	private IL10N $l10n;

	public function __construct(IUserManager $userManager, IGroupManager $groupManager, IL10N $l10n) {
		$this->userManager = $userManager;
		$this->groupManager = $groupManager;
		$this->l10n = $l10n;
	}

	/**
	 * @return list{boolean, string|null}
	 */
	public function canUserBeDeleted(IUser $user): array {
		if (!$this->userBackendAllowsDeletion($user)) {
			return [false, $this->l10n->t('User backend configuration does not allow deletion')];
		}
		if ($this->isOnlyUser()) {
			return [false, $this->l10n->t("You are the only user on the server, you can't delete your account.")];
		}
		if ($this->isOnlyAdmin($user)) {
			return [false, $this->l10n->t("You are the only admin on the server, you can't delete your account.")];
		}
		return [true, null];
	}

	/**
	 * Currently we only allow Database and Guests users to be deleted
	 */
	private function userBackendAllowsDeletion(IUser $user): bool {
		return in_array($user->getBackendClassName(), ['Database', 'Guests']);
	}

	private function isOnlyUser(): bool {
		return $this->userManager->countUsers() < 2;
	}

	private function isOnlyAdmin(IUser $user): bool {
		$adminGroup = $this->groupManager->get('admin');
		return $adminGroup && $adminGroup->count() < 2 && $this->groupManager->isAdmin($user->getUID());
	}
}
